/*
 * DecisionSubDataWrapper.h
 *
 *  Created on: Feb 14, 2024
 *      Author: yasmine
 */

#ifndef DMSubDataWrapper_WRAPPER_HEADER_H
#define DMSubDataWrapper_WRAPPER_HEADER_H

#include "sub/SubDataWrapper.h"
#include "dpc_dm/executable.h"


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class VED2DM_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::VED4DPC* _ved_dm = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _ved_dm = (dpc::VED4DPC*)ptr->Get();
    }

    void GetData(dpc::VED4DPC &data){
        if(_ved_dm)
        	data = *(_ved_dm);

    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMRoadInfo_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::RoadInfo* _road_info = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _road_info = (dpc::RoadInfo*)ptr->Get();
    }

    void GetData(dpc::RoadInfo &data){
    	if(_road_info)
    	{
    		data = *(_road_info);
    		 std::cout <<"################################################### roadInfo's ego lane id: " << +_road_info->car_position.ego_lane_id << std::endl;
    		 std::cout <<"###################################################  _road_info->lane_list.size(): " << +_road_info->lane_list.size() << std::endl;
//    		 std::cout <<"###################################################  _road_info->lane_list.front().corridor_list.size(): " << +_road_info->lane_list.front().corridor_list.size()<< std::endl;
    		 // std::cout <<"###################################################  _road_info->lane_list.front().corridor_list.front().reference_line.c0<: " << +_road_info->lane_list.front().corridor_list.front().reference_line.c0<< std::endl;

    	}

    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMRMHeader_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::RMHeader* _rm_header = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _rm_header = (dpc::RMHeader*)ptr->Get();
    }

    void GetData(dpc::RMHeader &data){
    	if(_rm_header)
        data = *(_rm_header);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMIndex_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::Index* _idx = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _idx = (dpc::Index*)ptr->Get();
    }

    void GetData(dpc::Index &data){
    	if(_idx)
        data = *(_idx);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMCarPosition_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::CarPosition* _car_pos = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _car_pos = (dpc::CarPosition*)ptr->Get();
    }

    void GetData(dpc::CarPosition &data){
    	if(_car_pos)
        data = *(_car_pos);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMLane_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::Lane* _lane = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _lane = (dpc::Lane*)ptr->Get();
    }

    void GetData(dpc::Lane &data){
    	if(_lane)
        data = *(_lane);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMLaneMarker_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::LaneMarker* _lane_marker = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _lane_marker = (dpc::LaneMarker*)ptr->Get();
    }

    void GetData(dpc::LaneMarker &data){
    	if(_lane_marker)
        data = *(_lane_marker);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMObject_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::Object* _object = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _object = (dpc::Object*)ptr->Get();
    }

    void GetData(dpc::Object &data){
    	if(_object)
        data = *(_object);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMRoadEdge_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::RoadEdge* _road_edge = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _road_edge = (dpc::RoadEdge*)ptr->Get();
    }

    void GetData(dpc::RoadEdge &data){
    	if(_road_edge)
        data = *(_road_edge);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMNeighborLane_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::NeighborLane* _neighbor_lane = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _neighbor_lane = (dpc::NeighborLane*)ptr->Get();
    }

    void GetData(dpc::NeighborLane &data){
    	if(_neighbor_lane)
        data = *(_neighbor_lane);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMCorridor_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::Corridor* _corridor = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _corridor = (dpc::Corridor*)ptr->Get();
    }

    void GetData(dpc::Corridor &data){
    	if(_corridor)
        data = *(_corridor);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMSamplingPoint_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::SamplingPoint* _sampling_point = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _sampling_point = (dpc::SamplingPoint*)ptr->Get();
    }

    void GetData(dpc::SamplingPoint &data){
    	if(_sampling_point)
        data = *(_sampling_point);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMLineColor_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::LineColor* _line_color = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _line_color = (dpc::LineColor*)ptr->Get();
    }

    void GetData(dpc::LineColor &data){
    	if(_line_color)
        data = *(_line_color);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMLineType_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::LineType* _line_type = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _line_type = (dpc::LineType*)ptr->Get();
    }

    void GetData(dpc::LineType &data){
    	if(_line_type)
        data = *(_line_type);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMObjectLocation_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::ObjectLocation* _object_loc = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _object_loc = (dpc::ObjectLocation*)ptr->Get();
    }

    void GetData(dpc::ObjectLocation &data){
    	if(_object_loc)
        data = *(_object_loc);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMSLPoint_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::SLPoint* _sl_point = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _sl_point = (dpc::SLPoint*)ptr->Get();
    }

    void GetData(dpc::SLPoint &data){
    	if(_sl_point)
        data = *(_sl_point);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMLaneControl_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::LaneControl* _lane_ctrl = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _lane_ctrl = (dpc::LaneControl*)ptr->Get();
    }

    void GetData(dpc::LaneControl &data){
    	if(_lane_ctrl)
        data = *(_lane_ctrl);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMLaneBoundary_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::LaneBoundary* _lane_bndry = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _lane_bndry = (dpc::LaneBoundary*)ptr->Get();
    }

    void GetData(dpc::LaneBoundary &data){
    	if(_lane_bndry)
        data = *(_lane_bndry);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMLineGeometry_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::LineGeometry* _line_geo = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _line_geo = (dpc::LineGeometry*)ptr->Get();
    }

    void GetData(dpc::LineGeometry &data){
    	if(_line_geo)
        data = *(_line_geo);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class EM2DMRoadEdgeSegment_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::RoadEdgeSegment* _road_edge_seg = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _road_edge_seg = (dpc::RoadEdgeSegment*)ptr->Get();
    }

    void GetData(dpc::RoadEdgeSegment &data){
    	if(_road_edge_seg)
        data = *(_road_edge_seg);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SLAM2DMAdjacentExtendLaneStruct_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::AdjacentExtendLaneStruct* _adj_ext_lane_str = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _adj_ext_lane_str = (dpc::AdjacentExtendLaneStruct*)ptr->Get();
    }

    void GetData(dpc::AdjacentExtendLaneStruct &data){
    	if(_adj_ext_lane_str)
        data = *(_adj_ext_lane_str);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SLAM2DMSMAStruct_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::SMAStruct* _sma_str = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _sma_str = (dpc::SMAStruct*)ptr->Get();
    }

    void GetData(dpc::SMAStruct &data){
    	if(_sma_str)
        data = *(_sma_str);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SLAM2DMWPExtendLaneStruct_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::WPExtendLaneStruct* _wp_ext_lane_str = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _wp_ext_lane_str = (dpc::WPExtendLaneStruct*)ptr->Get();
    }

    void GetData(dpc::WPExtendLaneStruct &data){
    	if(_wp_ext_lane_str)
        data = *(_wp_ext_lane_str);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SLAM2DMRemindWayPointStruct_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::RemindWayPointStruct* _reminder_wp_str = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _reminder_wp_str = (dpc::RemindWayPointStruct*)ptr->Get();
    }

    void GetData(dpc::RemindWayPointStruct &data){
    	if(_reminder_wp_str)
        data = *(_reminder_wp_str);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SLAM2DMControlWayPointStruct_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::ControlWayPointStruct* _ctrl_wp_str = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _ctrl_wp_str = (dpc::ControlWayPointStruct*)ptr->Get();
    }

    void GetData(dpc::ControlWayPointStruct &data){
    	if(_ctrl_wp_str)
        data = *(_ctrl_wp_str);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SLAM2DMMsgHeader_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::MsgHeader* _msg_header = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _msg_header = (dpc::MsgHeader*)ptr->Get();
    }

    void GetData(dpc::MsgHeader &data){
    	if(_msg_header)
        data = *(_msg_header);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SLAM2DMTransitLaneStruct_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::TransitLaneStruct* _transit_lane_str = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _transit_lane_str = (dpc::TransitLaneStruct*)ptr->Get();
//	    std::cout << "slam data: " << _transit_lane_str->transit_type << std::endl;
    }

    void GetData(dpc::TransitLaneStruct &data){
    	if(_transit_lane_str)
        data = *(_transit_lane_str);
    }
};


/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SLAM2DMControlWayPoints_SubDataWrapper :  public SubDataWrapper<DataType>


{
private:
    dpc::ControlWaypoints* _ctrl_wp = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    _ctrl_wp = (dpc::ControlWaypoints*)ptr->Get();
    }

    void GetData(dpc::ControlWaypoints &data){
    	if(_ctrl_wp)
        data = *(_ctrl_wp);
    }
};

/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class SM2Decision_SubDataWrapper :  public SubDataWrapper<DataType>
{
private:
	 dpc::SM4Decision*  _SM4Decision = nullptr;
public:
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	     _SM4Decision = (dpc::SM4Decision*)ptr->Get();
//        std::cout<<"Received SM2Decision data from SM is: "<<sample->sm_lfcmfrtavlbhvr<<std::endl;
    }

    void GetData( dpc::SM4Decision  &data){
    	if(_SM4Decision)
        data = *(_SM4Decision);
    }
};

/*-------------------------------------------------------------------------------------------------------------------------*/
template <typename DataType>
class CAN2DM_SubDataWrapper :  public SubDataWrapper<DataType>


{
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
	    auto sample = ptr->Get();
//	    std::cout<<"Received ses_en_sts data from CAN is: "<< sample->VCU_Info_1_Signals.ses_en_sts << std::endl;
    }
};


#endif /* DMPubDataWrapper_WRAPPER_HEADER_H */
